// TabTempl.cpp : implementation file
//

#include "stdafx.h"
#include "TabTempl.h"
#include "AfxPriv.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////
// CTabDocTemplate

IMPLEMENT_DYNAMIC(CTabDocTemplate, CMultiDocTemplate)

CTabDocTemplate::CTabDocTemplate (  UINT nIDResource, 
                                    CRuntimeClass* pDocClass, 
                                    CRuntimeClass* pFrameClass, 
                                    CRuntimeClass* pViewClass)
:   CMultiDocTemplate(nIDResource, pDocClass, 
                      pFrameClass, pViewClass)
{
}

CTabDocTemplate::~CTabDocTemplate (void)
{
#ifdef _DEBUG
    if (!m_docList.IsEmpty())
        TRACE1("Warning: destroying CTabDocTemplate with"
               "%d documents alive.\n",    m_docList.GetCount());
#endif
}

void CTabDocTemplate::CloseAllDocuments (BOOL)
{
    POSITION pos = GetFirstDocPosition();
    while (pos != NULL) {
        CDocument* pDoc = GetNextDoc(pos);
        pDoc->DeleteContents();
        if (pDoc->m_bAutoDelete)
            delete pDoc;
    }
}

///////////////////////////////////////////////////////////////////
// CTabDocTemplate commands

CDocument* CTabDocTemplate::OpenDocumentFile (LPCTSTR lpszPathName,
                                              BOOL bMakeVisible)
{
    CWaitCursor wait;
    CDocument* pDocument = CreateNewDocument();
    if (pDocument == NULL) {
        TRACE("CMultiDocTemplate::CreateNewDocument returns 0\n");
        AfxMessageBox(AFX_IDP_FAILED_TO_CREATE_DOC);
        return NULL;
    }
    ASSERT_VALID(pDocument);

    if (lpszPathName == NULL) {
        // create a new document - with default document name
        SetDefaultTitle(pDocument);
    } else {
        pDocument->SetPathName(lpszPathName);
    }

    BOOL bCreated = FALSE;
    CFrameWnd* pFrame = 0;
    if (1 == m_docList.GetCount()) {
        BOOL bAutoDelete = pDocument->m_bAutoDelete;
        pDocument->m_bAutoDelete = FALSE;   
        pFrame = CreateNewFrame(pDocument, NULL);
        bCreated = TRUE;
        pDocument->m_bAutoDelete = bAutoDelete;
    } else {
        pFrame = DYNAMIC_DOWNCAST(CFrameWnd, AfxGetMainWnd());
        ASSERT(pFrame);
        CCreateContext context;
        context.m_pCurrentFrame = 0;
        context.m_pCurrentDoc = pDocument;
        context.m_pNewViewClass = m_pViewClass;
        context.m_pNewDocTemplate = this;

        pFrame->LoadFrame(m_nIDResource, 
            WS_OVERLAPPEDWINDOW | FWS_ADDTOTITLE, 0, &context);
    }

    if (pFrame == NULL) {
        AfxMessageBox(AFX_IDP_FAILED_TO_CREATE_DOC);
        delete pDocument;       // explicit delete on error
        return NULL;
    }
    ASSERT_VALID(pFrame);

    if (lpszPathName == NULL) {
        if (!bMakeVisible)
            pDocument->m_bEmbedded = TRUE;

        if (!pDocument->OnNewDocument()) {
            // user must be alerted to what failed in OnNewDocument
            TRACE0("CDocument::OnNewDocument returned FALSE.\n");
            if (bCreated)
                pFrame->DestroyWindow();
            return NULL;
        }

        // it worked, now bump untitled count
        m_nUntitledCount++;
    } else {
        // open an existing document
        CWaitCursor wait;
        if (!pDocument->OnOpenDocument(lpszPathName)) {
            // user must be alerted to what failed
            TRACE0("CDocument::OnOpenDocument returned FALSE.\n");
            if (bCreated) {
                pFrame->DestroyWindow();
            }
            return NULL;
        }
    }

    CWinThread* pThread = AfxGetThread();
    if (pThread->m_pMainWnd == NULL)
        pThread->m_pMainWnd = pFrame;
    if (bCreated)
        InitialUpdateFrame(pFrame, pDocument, bMakeVisible);
    else
    {
        CView* pView = pFrame->GetActiveView();
        ASSERT(pView);
        pView->SendMessage(WM_INITIALUPDATE);
        pView->SendMessageToDescendants(WM_INITIALUPDATE, 
                                        0, 0, TRUE, TRUE);
        pFrame->OnUpdateFrameTitle(TRUE);
    }
    return pDocument;
}

void 
CTabDocTemplate::SetDefaultTitle (CDocument* pDocument)
{
    CString strDocName;
    if (GetDocString(strDocName, CDocTemplate::docName) &&
        !strDocName.IsEmpty()) {
        TCHAR szNum[8];
        wsprintf(szNum, _T("%d"), m_nUntitledCount+1);
        strDocName += szNum;
    } else {
        // use generic 'untitled' - ignore untitled count
        CString strUntitled(MAKEINTRESOURCE(AFX_IDS_UNTITLED));
        strDocName.Format("%s%d", strUntitled, m_nUntitledCount+1);
    }
    pDocument->SetTitle(strDocName);
}

