// TabWnd.cpp : implementation file
//

#include "stdafx.h"
#include "TabWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TARGET_NAME_LEN 30

//Utility function
UINT GetFileName (LPCTSTR lpszPathName, 
                  LPTSTR lpszTitle, UINT nMax)
{
    ASSERT(lpszTitle == NULL ||
        AfxIsValidAddress(lpszTitle, _MAX_FNAME));
    ASSERT(AfxIsValidString(lpszPathName));

    // always capture the complete file name 
    // including extension (if present)
    LPTSTR lpszTemp = const_cast<LPTSTR>(lpszPathName);
    for ( LPCTSTR lpsz = lpszPathName; 
          *lpsz != '\0'; lpsz = _tcsinc(lpsz))
    {
        // remember last directory/drive separator
        if (*lpsz == '\\' || *lpsz == '/' || *lpsz == ':')
            lpszTemp = const_cast<LPTSTR>(_tcsinc(lpsz));
    }

    // lpszTitle can be NULL which just returns 
    // the number of bytes
    if (lpszTitle == NULL)
        return lstrlen(lpszTemp) + 1;

    // otherwise copy it into the buffer provided
    lstrcpyn(lpszTitle, lpszTemp, nMax);
    return 0;
}

////////////////////////////////////////////////////////////////
// CTabbedFrameWnd

IMPLEMENT_DYNCREATE(CTabbedFrameWnd, CFrameWnd)

CTabbedFrameWnd::CTabbedFrameWnd (void)
:m_nActiveTab(-1)
, m_pActiveView(0)
, m_pToolTip(new CToolTipCtrl())
{
}

CTabbedFrameWnd::~CTabbedFrameWnd (void)
{
    delete m_pToolTip;
}

BEGIN_MESSAGE_MAP(CTabbedFrameWnd, CFrameWnd)
    //{{AFX_MSG_MAP(CTabbedFrameWnd)
    ON_WM_SIZE()
    ON_NOTIFY(TCN_SELCHANGE, AFX_IDW_PANE_FIRST, OnTabSelChange)
    ON_WM_CLOSE()
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

////////////////////////////////////////////////////////////////
// CTabbedFrameWnd message handlers
void CTabbedFrameWnd::OnTabSelChange (NMHDR*, 
                                      LRESULT* pResult)
{
    *pResult = 0;
    SetActiveView(m_TabCtrl.GetCurSel());
    OnUpdateFrameTitle(TRUE);
    Invalidate();
}

void CTabbedFrameWnd::OnClose (void) 
{
    if (m_lpfnCloseProc != NULL && !(*m_lpfnCloseProc)(this))
        return;

    CWinApp* pApp = AfxGetApp();
    if (pApp != NULL && pApp->m_pMainWnd == this) {
        // attempt to save all documents
        if (!pApp->SaveAllModified())
            return;     // don't close it

        // hide the application's windows before closing documents
        pApp->HideApplication();

        // close all documents first
        pApp->CloseAllDocuments(FALSE);

        // don't exit if there are outstanding component objects
        if (!AfxOleCanExitApp()) {
            // take user out of control of the app
            AfxOleSetUserCtrl(FALSE);

            // don't destroy the main window and close down yet
            //  (there are outstanding component (OLE) objects)
            return;
        }

        // there are cases where destroying the documents may 
        // destroy the main window of the application.
        if (!afxContextIsDLL && pApp->m_pMainWnd == NULL) {
            AfxPostQuitMessage(0);
            return;
        }
    }

    // then destroy the window
    DestroyWindow();
}

////////////////////////////////////////////////////////////////
// CTabbedFrameWnd virtual overrides
BOOL 
CTabbedFrameWnd::PreCreateWindow (CREATESTRUCT& cs)
{
    if( !CFrameWnd::PreCreateWindow(cs) )
        return FALSE;

    static BOOL bRegistered = FALSE; // flag for first-time init
    if (!bRegistered) 
    {
        // This is the first time the frame is being created.
        // Need to register the frame class.          
       WNDCLASSEX wc;
        wc.cbSize = sizeof(WNDCLASSEX);

        // Get class information for MFC default view.
        // The classname is in cs.lpszClass
        GetClassInfoEx(AfxGetInstanceHandle(), cs.lpszClass, &wc);

        // Modify name and style            
        wc.lpszClassName = _T("SynergexTabFrame");
        VERIFY(RegisterClassEx(&wc));  
        bRegistered = TRUE;
    }

    cs.lpszClass = _T("SynergexTabFrame");
    cs.dwExStyle &= ~WS_EX_CLIENTEDGE;

    return TRUE;
}

BOOL CTabbedFrameWnd::OnCreateClient (LPCREATESTRUCT, 
                                      CCreateContext* pContext) 
{
    if (!m_TabCtrl.Create(
        WS_VISIBLE | WS_CHILD | TCS_TOOLTIPS | TCS_BOTTOM, 
        CRect(0, 0, 0, 0), this, AFX_IDW_PANE_FIRST)) 
        return FALSE;

    CFont* pFont = CFont::FromHandle(static_cast<HFONT>(
        ::GetStockObject(DEFAULT_GUI_FONT)));
    m_TabCtrl.SetFont(pFont);

    m_pToolTip->Create(&m_TabCtrl);
    m_TabCtrl.SetToolTips(m_pToolTip);

    // default create client will create a view if asked for it
    if (pContext != NULL && pContext->m_pNewViewClass != NULL)
        if (!AddView(pContext))
            return FALSE;
    return TRUE;
}

BOOL CTabbedFrameWnd::LoadFrame (UINT nIDResource, 
                                 DWORD dwDefaultStyle, 
                                 CWnd* pParentWnd, 
                                 CCreateContext* pContext) 
{
    if (m_arrViews.GetSize() == 0)
        return CFrameWnd::LoadFrame(
            nIDResource, dwDefaultStyle, pParentWnd, pContext);
    return AddView(pContext);
}

void CTabbedFrameWnd::RecalcLayout (BOOL bNotify) 
{
    CFrameWnd::RecalcLayout(bNotify);
    
    CRect rect;
    m_TabCtrl.GetClientRect(rect);
    m_TabCtrl.AdjustRect(FALSE, rect);

    //Display rectangle is always 2 pixels too close to the tabs.
    BOOL bBottom = m_TabCtrl.GetStyle() & TCS_BOTTOM;
    BOOL bVertical = m_TabCtrl.GetStyle() & TCS_VERTICAL;

    if (bBottom && bVertical) 
        rect.right -= 2;
    else if (bBottom)
        rect.bottom -= 2;
    else if (bVertical)
        rect.left += 2;
    else
        rect.top += 2;
    
    for (int n = 0; n < m_arrViews.GetSize(); n++)
        m_arrViews.GetAt(n)->MoveWindow(rect);
}

void CTabbedFrameWnd::OnUpdateFrameTitle (BOOL bAddToTitle)
{
    CFrameWnd::OnUpdateFrameTitle(bAddToTitle);

    CDocument* pDoc = m_pActiveView->GetDocument();

    CString strDocName(pDoc->GetPathName());
    CString strLabel(pDoc->GetTitle());
    if (!strDocName.IsEmpty())
        strLabel = GetShortName(strDocName, TARGET_NAME_LEN); 

    TC_ITEM tci;
    tci.mask = TCIF_TEXT;
    tci.pszText = strLabel.GetBuffer(strLabel.GetLength() + 1);
    m_TabCtrl.SetItem(m_nActiveTab, &tci);
    Invalidate();
}

////////////////////////////////////////////////////////////////
// CTabbedFrameWnd implementation
BOOL CTabbedFrameWnd::AddView (CCreateContext* pContext)
{
    ASSERT_VALID(this);
    ASSERT(pContext);

    CWaitCursor wait;

    CRuntimeClass* pViewClass = pContext->m_pNewViewClass;
    ASSERT(pViewClass);
    ASSERT(pViewClass->IsDerivedFrom(RUNTIME_CLASS(CWnd)));
    ASSERT(AfxIsValidAddress(pViewClass, 
              sizeof(CRuntimeClass), FALSE));
    
    CString strDocName(pContext->m_pCurrentDoc->GetPathName());
    CString strLabel(pContext->m_pCurrentDoc->GetTitle());
    if (!strDocName.IsEmpty())
        strLabel = GetShortName(strDocName, TARGET_NAME_LEN); 

    CWnd* pWnd;
    TRY
    {
        pWnd = dynamic_cast<CWnd*>(pViewClass->CreateObject());
        ASSERT(pWnd);
        if (!pWnd)
            AfxThrowMemoryException();
    }
    CATCH_ALL(e)
    {
        return FALSE;
    }
    END_CATCH_ALL

    ASSERT_KINDOF(CWnd, pWnd);
    ASSERT(!pWnd->m_hWnd);

    DWORD dwStyle = AFX_WS_DEFAULT_VIEW;
    if (!pWnd->Create(0, 0, dwStyle, CRect(0, 0, 0, 0), 
                      &m_TabCtrl, 0, pContext))
        return FALSE;

    m_pActiveView = dynamic_cast<CView*>(pWnd);

    m_arrViews.Add(pWnd);

    if (0 == (m_arrViews.GetSize() - 1))
        m_pToolTip->AddTool(&m_TabCtrl, strLabel);
    else
        m_pToolTip->AddTool(&m_TabCtrl, strLabel, 
                            CRect(0, 0, 0, 0), 
                            (m_arrViews.GetSize() - 1));
    
    int nViews = m_arrViews.GetSize();

    TC_ITEM tci;
    tci.mask = TCIF_TEXT | TCIF_IMAGE;
    tci.pszText = strLabel.GetBuffer(1);
    tci.iImage = nViews - 1;
    m_TabCtrl.InsertItem(nViews, &tci);
    strLabel.ReleaseBuffer(-1);

    if (nViews > 1) 
        RecalcLayout();
    SetActiveView(nViews - 1);

    return TRUE;
}

void CTabbedFrameWnd::SetActiveView (int nNewTab)
{
    ASSERT_VALID(this);
    ASSERT(nNewTab >= 0);

    if (nNewTab != -1 && nNewTab != m_nActiveTab) {
        CWnd* pNewView = m_arrViews[nNewTab];
        CWnd* pOldView = 0;

        if (m_nActiveTab != -1) {
            pOldView = m_arrViews[m_nActiveTab];
            pOldView->EnableWindow(FALSE);
            pOldView->ShowWindow(SW_HIDE);
        }
        pNewView->EnableWindow(TRUE);
        pNewView->ShowWindow(SW_SHOW);
        pNewView->SetFocus();

        m_pActiveView = dynamic_cast<CView*>(pNewView);
        ASSERT(m_pActiveView);

        SetActiveView(m_pActiveView);

        m_nActiveTab = nNewTab;
        m_TabCtrl.SetCurSel(m_nActiveTab);
        OnUpdateFrameTitle(TRUE);
    }
}

BOOL CTabbedFrameWnd::OnCloseCurrentDoc (void)
{
    return OnCloseDoc(m_TabCtrl.GetCurSel());
}

BOOL CTabbedFrameWnd::OnCloseDoc (int nTab)
{
    CWnd* pWnd = m_arrViews[nTab];
    CView* pView = dynamic_cast<CView*>(pWnd);
    ASSERT(pView);

    int nNewTab = 0;
    if (m_arrViews.GetSize() > 1) {
        if (nTab > 0)
            nNewTab = nTab - 1;
        else
            nNewTab = m_arrViews.GetSize() - 1;
    } else {
        CCreateContext create;
        create.m_pCurrentDoc = 0;
        create.m_pCurrentFrame = 0;
        create.m_pLastView = 0;
        create.m_pNewDocTemplate = 0;
        create.m_pNewViewClass = pView->GetRuntimeClass();
        AddView(&create);
        nNewTab = m_arrViews.GetSize() - 1;
    }

    SetActiveView(nNewTab);

    CDocument* pDoc = pView->GetDocument();
    if (pDoc) {
        if (!pDoc->SaveModified())
            return FALSE;
        pDoc->DeleteContents();
    }
    pView->DestroyWindow();
    m_arrViews.RemoveAt(nTab);
    m_TabCtrl.DeleteItem(nTab);
    return TRUE;
}

CString CTabbedFrameWnd::GetShortName (const CString& strName, 
                                       UINT nTargetLen)
{
    CString strCurDir("");
    GetCurrentDirectory(_MAX_PATH, strCurDir.GetBuffer(_MAX_PATH));
    strCurDir.ReleaseBuffer();
    ASSERT(!strCurDir.IsEmpty());
    strCurDir += _T("\\");
    int nCurDir = strCurDir.GetLength();

    CString strRet(strName);       

    // nLenDir is the length of the directory part of the full path
    int nLenDir = strRet.GetLength() 
                - (GetFileName(strRet, 0, 0) - 1);
    BOOL bSameDir = FALSE;
    if (nLenDir == nCurDir)
    {
        bSameDir = (strCurDir == strRet.Left(nLenDir));
    }
    // copy the full path, otherwise abbreviate the name
    if (bSameDir)
    {
        // copy file name only since directories are same
        TCHAR szTemp[_MAX_PATH];
        ::GetFileTitle(strName.Right(
            strName.GetLength() - nCurDir), 
            szTemp, sizeof(szTemp)/sizeof(TCHAR));
        strRet = szTemp;
    }
    else if (nTargetLen != -1)
    {
        // strip the extension if the system calls for it
        TCHAR szTemp[_MAX_PATH];
        ::GetFileTitle(strName.Right(
            strName.GetLength() - nLenDir), 
            szTemp, sizeof(szTemp)/sizeof(TCHAR));
        strRet = strRet.Left(nLenDir);
        strRet += szTemp;

        // abbreviate name based on what will fit in limited space
        strRet = ShortenName(strRet, nTargetLen);
    }
    return strRet;
}

CString CTabbedFrameWnd::ShortenName (const CString& strFullName, 
                                      UINT nTarget)
{
    if (nTarget >= static_cast<UINT>(strFullName.GetLength()))
        return strFullName;
    
    UINT nFileNameLen = GetFileName(strFullName, 0, 0) - 1;
    CString strFileName = strFullName.Right(nFileNameLen);

    if (nTarget <= nFileNameLen)
    {
        return strFileName;
    }

    // Calculate the length of the volume name.  Normally, this is
    // two characters (e.g., "C:", "D:", etc.), but for a UNC name,
    // it could be more (e.g., "\\server\share").
    //
    // If cchMax isn't enough to hold at least 
    // <volume_name>\...\<base_name>, the result is the base name.
    CString strVolName = strFullName.Left(2);
    UINT nVolNameLen = 2;
    if (strVolName == _T("\\\\"))
    {
        int nNextSlash = strFullName.Find(_T('\\'), nVolNameLen);
        ASSERT(nNextSlash != -1);
        
        strVolName += strFullName.Mid(nVolNameLen, 
                                      nNextSlash - nVolNameLen);
        nVolNameLen = nNextSlash;
    }
    ASSERT(strFullName.GetAt(nVolNameLen) == _T('\\'));

    if (strFullName.GetLength() - strFileName.GetLength() > 3)
    {
        int nNextSlash = strFullName.Find(_T('\\'), 
            nVolNameLen + 1);
        ASSERT(nNextSlash != -1);
        
        strVolName += strFullName.Mid(nVolNameLen, 
                                      nNextSlash - nVolNameLen);
        nVolNameLen = nNextSlash;
    }
    ASSERT(strFullName.GetAt(nVolNameLen) == _T('\\'));

    if (nTarget < nVolNameLen + 5 + nFileNameLen)
    {
        return strFileName;
    }

    UINT nCurrent = nVolNameLen;
    while (nVolNameLen + 4 + (strFullName.GetLength() - nCurrent)
           > nTarget)
    {
        nCurrent = strFullName.Find(_T('\\'), nCurrent + 1);
        ASSERT(nCurrent != -1);
    }

    // Form the resultant string and we're done.
    CString strRet(strVolName);
    strRet += _T("\\...");
    strRet += strFullName.Right(strFullName.GetLength() 
                                - nCurrent);
    return strRet;
}


