#include <stdio.h>
#include <winsock2.h>

/* for the well-formed request compile with: */
/* cl dack.c -link ws2_32.lib */

/* for the penalty case compile with: */
/* cl -D DACK_PENALTY dack.c -link ws2_32.lib */

int netStartup(void)
{
    WSADATA actWSA;
    WORD wVersionRequested;
    int iError;

    wVersionRequested=MAKEWORD(2,0);

    /* let's start Winsock services */
    iError=WSAStartup(wVersionRequested,&actWSA);
    if(iError!=0)
    {
        fprintf(stderr,"Error starting up Winsock:%d\n",iError);
        exit(1);
    }

    /* test Winsock Version (we want 2.0) */
    if(LOBYTE(actWSA.wVersion )!= 2 || HIBYTE(actWSA.wVersion )!=0)
    { 
        /* sorry, not our version! */
        WSACleanup();
        fprintf(stderr,"Wrong Winsock version!\n"); 
        exit(1);
    }
    return 1;
}

int netShutdown(void)
{
    /* stop Winsock */
    if(WSACleanup()==WSAEINPROGRESS)
    {
        WSACancelBlockingCall();
        WSACleanup();
    }
    return 0;
}

void printUsage(char** argv)
{
    fprintf(stderr,"Wrong number of parameters.\nUsage:\n");
    fprintf(stderr,"client side: %s c host portno\n",argv[0]);
    fprintf(stderr,"server side: %s s portno\n",argv[0]);
    exit(2);
}

int readSocket(SOCKET sock,char* buffer,int length)
{
    int received;
    received=recv(sock,buffer,length,0);
    if(received<0)
    {
        fprintf(stderr,"Error at reading from socket:%d\n",WSAGetLastError());
    }
    return received;
}

int writeSocket(SOCKET sock,char* buffer,int length)
{
    int totalWritten;
    int written;
    totalWritten=written=0;
    while(totalWritten<length&&written>=0)
    {
        written=send(sock,buffer,length,0);
        if(written<0)
        {
            totalWritten=-1;
            fprintf(stderr,"Error at writing to socket:%d\n",WSAGetLastError());
        }
        totalWritten+=written;
    }
    return totalWritten;
}

void doClientPart(char* host,char* port)
{
    struct sockaddr_in name;
    unsigned long addr;
    SOCKET sock;
    char sendBuffer[4096];
    DWORD startTime;
    DWORD stopTime;
    int written;
    int read;
    int i;
#ifdef BRUTE_FORCE_CURE
    int ON;
#endif

    memset(&name,0,sizeof(name));
    name.sin_family=AF_INET;
    addr=inet_addr(host);
    if(addr==INADDR_NONE)
    {
        fprintf(stderr,"Please specify an IP address (e.g. 127.0.0.1)\n");
        exit(4);
    }
    name.sin_addr.s_addr=addr ;
    name.sin_port=htons((unsigned short)atoi(port));

    sock=socket(AF_INET,SOCK_STREAM,0);
    if(sock==SOCKET_ERROR)
    {
        fprintf(stderr,"Error at creating socket:%d\n",WSAGetLastError());
        exit(4);
    }

    if(connect(sock,(struct sockaddr*)&name,sizeof(name))==SOCKET_ERROR)
    {
        fprintf(stderr,"Error at connecting to server:%d\n",WSAGetLastError());
        exit(4);
    }

#ifdef BRUTE_FORCE_CURE
    ON=1;
    if(setsockopt(sock,IPPROTO_TCP,TCP_NODELAY,(const char*)&ON,sizeof(int))==SOCKET_ERROR)
    {
        printf("Switching off Nagle!");
        fprintf(stderr,"Error at switching off Nagle:%d\n",WSAGetLastError());
        exit(4);
    }
#endif

    memset(sendBuffer,'A',4096);
    for(i=0;i<100;i++)
    {
        startTime=GetTickCount();
#ifdef DACK_PENALTY
        written=writeSocket(sock,sendBuffer,1024);
        written=writeSocket(sock,sendBuffer,1024);
#else
        written=writeSocket(sock,sendBuffer,2048);
#endif
        read=readSocket(sock,sendBuffer,4096);
        stopTime=GetTickCount();
        printf("Time elapsed (milliseconds):%ld\n",stopTime-startTime);
    }

    // now shut down the connection!
    closesocket(sock);
}

void doServerPart(char* port)
{
    struct sockaddr_in name;
    SOCKET listenSock;
    SOCKET acceptedSock;
    int optval;
    int optlen;
    struct sockaddr acceptMem;
    struct sockaddr* acceptAddr;
    int addrLength;
    int running;
    int active;
    char receiveBuffer[4096];
    int read;
    int totalRead;

    printf("listening on port:%d\n",atoi(port));
    memset(&name,0,sizeof(name));
    name.sin_family=AF_INET;
    name.sin_port=htons((unsigned short)atoi(port));
    name.sin_addr.s_addr=htonl(INADDR_ANY);
    listenSock=socket(AF_INET,SOCK_STREAM,0);
    if(listenSock==SOCKET_ERROR)
    {
        fprintf(stderr,"Error at creating listening socket:%d\n",WSAGetLastError());
        exit(3);
    }

    optval=1;
    optlen=sizeof(int);
    if(setsockopt(listenSock,SOL_SOCKET,SO_REUSEADDR,(char*)&optval,optlen)==SOCKET_ERROR)
    {
        fprintf(stderr,"Error at setting socket options:%d\n",WSAGetLastError());
        exit(3);
    }
   
    if(bind(listenSock,(struct sockaddr*)&name,sizeof(name))==SOCKET_ERROR)
    {
        fprintf(stderr,"Error at binding socket:%d\n",WSAGetLastError());
        exit(3);
    }
  
    if(listen(listenSock,SOMAXCONN)==SOCKET_ERROR)
    {
        fprintf(stderr,"Error at listening on socket:%d\n",WSAGetLastError());
        exit(3);
    }
    printf("Server is now listening for incoming connections\n");

    running=1;
    while(running)
    {
        acceptAddr=&acceptMem;
        addrLength=sizeof(acceptMem);
        acceptedSock=accept(listenSock,acceptAddr,&addrLength);
        if(acceptedSock==SOCKET_ERROR)
        {
            fprintf(stderr,"Error at accepting socket:%d\n",WSAGetLastError());
            exit(3);
        }
        printf("Client connection has been accepted\n");

        /* this loop is used to answer requests until the client closes */
        /* the connection (active is set to 0) */
        active=1;
        while(active)
        {
            totalRead=0;
            /* this loop ensures that always the whole request is read */
            while(totalRead<2048)
            {
                read=readSocket(acceptedSock,receiveBuffer,4096);
                if(read>0)
                    totalRead+=read;
                else if(read<0)
                {
                    printf("Client has closed the connection non gracefully!\n");
                    active=0;
                    break;
                }
                else if(read==0)
                {
                    printf("Client has gracefully closed the connection!\n");
                    active=0;
                    break;
                }
            }
      
            /* the server sends answers only if the connection is */
            /* still active */
            if(active)
                writeSocket(acceptedSock,"OK",2);
        }
        closesocket(acceptedSock);
        printf("Server has finished this session\n");
    }
    printf("Server terminating!\n");
}

int main(int argc,char** argv)
{
    if(argc<3)
        printUsage(argv);

    netStartup();

    /* check if we are in server or client mode */
    if(*argv[1]=='c')
    {
        if(argc<4)
            printUsage(argv);
        doClientPart(argv[2],argv[3]);
    }
    else if(*argv[1]=='s')
    {
        doServerPart(argv[2]);
    }
    else
        printUsage(argv);

    netShutdown();
    return 0;
}
