// jobobj.c

#define _WIN32_WINNT 0x0500
#include <windows.h>
#include "jobobj.h"

VOID GetLimits(HWND, HANDLE);
VOID SetLimits(HWND, HANDLE);
void DumpJobAccountingInfo(HWND, HANDLE);
int CALLBACK JobDlgProc(HWND, UINT, WPARAM, LPARAM);
int CALLBACK LimitsDlgProc(HWND, UINT, WPARAM, LPARAM);

HANDLE hInst;
TCHAR sz[MAX_PATH];
BOOL bInit = TRUE;

//-----------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine, int nCmdShow)
{
    HANDLE hJob;

    hInst = hInstance;
    hJob = CreateJobObject(NULL, NULL);
    if (hJob) {
        AssignProcessToJobObject(hJob, GetCurrentProcess());
        DialogBoxParam(hInst, MAKEINTRESOURCE(DLG_JOBOBJ), NULL, 
                       JobDlgProc, (LPARAM)hJob);
        CloseHandle(hJob);
    }
    return 0;
}

//-----------------------------------------------------------------
int CALLBACK JobDlgProc(HWND hDlg, UINT Message, WPARAM wParam,
                        LPARAM lParam)
{
    HANDLE hJob;
    STARTUPINFO si;
    PROCESS_INFORMATION pi;
    DWORD dwFlags = 0;

    switch (Message) {

        case WM_INITDIALOG:
            SetWindowLong(hDlg, GWL_USERDATA, lParam);
            DumpJobAccountingInfo(GetDlgItem(hDlg, ID_QUERY_INFO), 
                                  (HANDLE)lParam);
            return TRUE;

        case WM_COMMAND:
            hJob = (HANDLE)GetWindowLong(hDlg, GWL_USERDATA);

            switch (LOWORD(wParam)) {
                case IDOK:
                    EndDialog(hDlg, TRUE);
                    return TRUE;

                case ID_TERMINATE:
                    TerminateJobObject(hJob, TRUE);
                    break;

                case ID_QUERY_REFRESH:
                   DumpJobAccountingInfo(
                        GetDlgItem(hDlg, ID_QUERY_INFO), hJob);
                    break;

                case ID_CREATEPROC:
                    if (IsDlgButtonChecked(hDlg, ID_BREAKAWAY)) {
                        dwFlags = CREATE_BREAKAWAY_FROM_JOB; 
                    }
                    memset(&si, 0, sizeof(si));
                    si.cb = sizeof(si);

                    if (!CreateProcess("childp.exe",NULL,NULL,NULL,
                                       FALSE,dwFlags,NULL,NULL,
                                       &si, &pi)) {
                        wsprintf(sz, "Error (%d)", GetLastError());
                        MessageBox(hDlg, sz, "JobObj", MB_OK);
                    }
                    break;

                case ID_LIMITS:
                    DialogBoxParam(hInst,MAKEINTRESOURCE(DLG_LIMITS),
                                   NULL,LimitsDlgProc,(LPARAM)hJob);
                    break;
            }

        default:
            return FALSE;
    }

}

//-----------------------------------------------------------------
int CALLBACK LimitsDlgProc(HWND hDlg, UINT Message, WPARAM wParam,
                           LPARAM lParam)
{
    HANDLE hJob;

    switch (Message) {

        case WM_INITDIALOG:
            SetWindowLong(hDlg, GWL_USERDATA, lParam);
            GetLimits(hDlg, (HANDLE)lParam);
            return TRUE;

        case WM_COMMAND:

            switch (LOWORD(wParam)) {
                case IDOK:
                    hJob = (HANDLE)GetWindowLong(hDlg,GWL_USERDATA);
                    SetLimits(hDlg, hJob);
                    EndDialog(hDlg, TRUE);
                    break;

                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    break;
            }

        default:
            return FALSE;
    }
}

//-----------------------------------------------------------------
VOID GetLimits(HWND hDlg, HANDLE hJob)
{
    BYTE Data[1024];
    PJOBOBJECT_BASIC_LIMIT_INFORMATION  pBasic;
    PJOBOBJECT_EXTENDED_LIMIT_INFORMATION pExt;
    PJOBOBJECT_BASIC_UI_RESTRICTIONS pUI;

    if (QueryInformationJobObject(hJob, 
            JobObjectExtendedLimitInformation, &Data, 
            sizeof(JOBOBJECT_EXTENDED_LIMIT_INFORMATION),NULL)) {

        pExt = (PJOBOBJECT_EXTENDED_LIMIT_INFORMATION)&Data;
        pBasic = &pExt->BasicLimitInformation;

        if (bInit) {    // use defaults
            pBasic->PerProcessUserTimeLimit.LowPart = 30000000;
            pBasic->PerJobUserTimeLimit.LowPart = 90000000; 
            pBasic->MinimumWorkingSetSize = 204800; 
            pBasic->MaximumWorkingSetSize = 1413120; 
            pBasic->ActiveProcessLimit = 5; 
            pExt->ProcessMemoryLimit = 0x6400000;    // 100MG
            pExt->JobMemoryLimit = 0x12C00000;       // 300MG
            bInit = FALSE;
        }

        SetDlgItemInt(hDlg, ID_PROCESSUSERTIMELIMIT,
                pBasic->PerProcessUserTimeLimit.LowPart, FALSE);
        SetDlgItemInt(hDlg, ID_JOBUSERTIMELIMIT,
                pBasic->PerJobUserTimeLimit.LowPart, FALSE);
        SetDlgItemInt(hDlg, ID_FLAGS, pBasic->LimitFlags, FALSE);
        SetDlgItemInt(hDlg, ID_MINWORKINGSETSIZE,
                pBasic->MinimumWorkingSetSize, FALSE);
        SetDlgItemInt(hDlg, ID_MAXWORKINGSETSIZE,
                pBasic->MaximumWorkingSetSize, FALSE);
        SetDlgItemInt(hDlg, ID_ACTIVEPROCESSLIMIT,
                pBasic->ActiveProcessLimit, FALSE);
        SetDlgItemInt(hDlg, ID_AFFINITY, pBasic->Affinity, FALSE);
        SetDlgItemInt(hDlg, ID_PRIORITYCLASS,
                pBasic->PriorityClass, FALSE);
        SetDlgItemInt(hDlg, ID_SCHEDULINGCLASS,
                pBasic->SchedulingClass, FALSE);

        SetDlgItemInt(hDlg, ID_PROCESSMEMLIMIT,
                pExt->ProcessMemoryLimit, FALSE);
        SetDlgItemInt(hDlg, ID_JOBMEMLIMIT,
                pExt->JobMemoryLimit, FALSE);
        SetDlgItemInt(hDlg, ID_PEAKPROCMEMUSED,
                pExt->PeakProcessMemoryUsed, FALSE);
        SetDlgItemInt(hDlg, ID_PEAKJOBMEMUSED,
                pExt->PeakJobMemoryUsed, FALSE);
    }
    
    if (QueryInformationJobObject(hJob,JobObjectBasicUIRestrictions,
            &Data,sizeof(JOBOBJECT_BASIC_UI_RESTRICTIONS),NULL)) {
        pUI = (PJOBOBJECT_BASIC_UI_RESTRICTIONS)&Data;
        SetDlgItemInt(hDlg, ID_UIRESTRICTIONS,
                pUI->UIRestrictionsClass, FALSE);
    }
    return;
}

//-----------------------------------------------------------------
VOID SetLimits(HWND hDlg, HANDLE hJob)
{
    JOBOBJECT_EXTENDED_LIMIT_INFORMATION Ext;
    JOBOBJECT_BASIC_UI_RESTRICTIONS UI;

    Ext.BasicLimitInformation.PerProcessUserTimeLimit.HighPart = 0;
    Ext.BasicLimitInformation.PerJobUserTimeLimit.HighPart = 0;
    Ext.BasicLimitInformation.PerProcessUserTimeLimit.LowPart = 
            GetDlgItemInt(hDlg, ID_PROCESSUSERTIMELIMIT, NULL,FALSE);
    Ext.BasicLimitInformation.PerJobUserTimeLimit.LowPart = 
            GetDlgItemInt(hDlg, ID_JOBUSERTIMELIMIT, NULL, FALSE);
    Ext.BasicLimitInformation.MaximumWorkingSetSize = 
            GetDlgItemInt(hDlg, ID_MAXWORKINGSETSIZE, NULL, FALSE);
    Ext.BasicLimitInformation.MinimumWorkingSetSize = 
            GetDlgItemInt(hDlg, ID_MINWORKINGSETSIZE, NULL, FALSE);
    Ext.BasicLimitInformation.ActiveProcessLimit = 
            GetDlgItemInt(hDlg, ID_ACTIVEPROCESSLIMIT, NULL, FALSE);
    Ext.ProcessMemoryLimit = 
            GetDlgItemInt(hDlg, ID_PROCESSMEMLIMIT, NULL, FALSE);
    Ext.JobMemoryLimit = 
            GetDlgItemInt(hDlg, ID_JOBMEMLIMIT, NULL, FALSE);

    Ext.BasicLimitInformation.LimitFlags = 
       JOB_OBJECT_LIMIT_PROCESS_MEMORY | JOB_OBJECT_LIMIT_JOB_MEMORY
     | JOB_OBJECT_LIMIT_PROCESS_TIME | JOB_OBJECT_LIMIT_JOB_TIME 
     | JOB_OBJECT_LIMIT_WORKINGSET | JOB_OBJECT_LIMIT_ACTIVE_PROCESS
     | JOB_OBJECT_LIMIT_BREAKAWAY_OK;

    if (!SetInformationJobObject(hJob, 
           JobObjectExtendedLimitInformation, &Ext, sizeof(Ext))) {
        wsprintf(sz, "Set ExtLimit Error (%d)", GetLastError());
        MessageBox(hDlg, sz, "JobObj", MB_OK);
    }

    UI.UIRestrictionsClass = 
            GetDlgItemInt(hDlg, ID_UIRESTRICTIONS, NULL, FALSE);

    if (!SetInformationJobObject(hJob,JobObjectBasicUIRestrictions,
                &UI, sizeof(UI))) {
        wsprintf(sz, "Set UILimit Error (%x)", GetLastError());
        MessageBox(hDlg, sz, "JobObj", MB_OK);
    }
    return;
}

//-----------------------------------------------------------------
void DumpJobAccountingInfo(HWND hListBox, HANDLE hJob)
{
    BYTE Data[1024], i;
    PJOBOBJECT_BASIC_ACCOUNTING_INFORMATION pBasic;
    PJOBOBJECT_BASIC_PROCESS_ID_LIST pProc;

    SendMessage(hListBox, LB_RESETCONTENT, 0, 0);

    if (QueryInformationJobObject(
            hJob, JobObjectBasicAccountingInformation, &Data, 
            sizeof(JOBOBJECT_BASIC_ACCOUNTING_INFORMATION),NULL)) {

        pBasic = (PJOBOBJECT_BASIC_ACCOUNTING_INFORMATION)&Data;
        
        wsprintf(sz, "TotalUserTime=%x %x", 
                 pBasic->TotalUserTime.HighPart,
                 pBasic->TotalUserTime.LowPart);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        wsprintf(sz, "TotalKernelTime=%x %x", 
                 pBasic->TotalKernelTime.HighPart,
                 pBasic->TotalKernelTime.LowPart);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        wsprintf(sz, "ThisPeriodTotalUserTime=%x %x",
                 pBasic->ThisPeriodTotalUserTime.HighPart,
                 pBasic->ThisPeriodTotalUserTime.LowPart);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        wsprintf(sz, "ThisPeriodTotalKernelTime=%x %x",
                 pBasic->ThisPeriodTotalKernelTime.HighPart,
                 pBasic->ThisPeriodTotalKernelTime.LowPart);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        wsprintf(sz, "TotalPageFaultCount=%x",
                 pBasic->TotalPageFaultCount);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        wsprintf(sz, "TotalProcesses=%x",pBasic->TotalProcesses);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        wsprintf(sz, "ActiveProcesses=%x",pBasic->ActiveProcesses);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        wsprintf(sz, "TotalTerminatedProcesses=%x",
                 pBasic->TotalTerminatedProcesses);
        SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
    }

    if (QueryInformationJobObject(hJob,JobObjectBasicProcessIdList, 
                                  &Data, 1024, NULL)) {
        pProc = (PJOBOBJECT_BASIC_PROCESS_ID_LIST)&Data;
        for (i=0; i < pProc->NumberOfProcessIdsInList; i++) {
            wsprintf(sz,"PID[%d]=%d",i,pProc->ProcessIdList[i]);
            SendMessage(hListBox, LB_ADDSTRING, 0, (LPARAM)sz);
        }
    }
    return;
}
